@@ -40,34 +40,52 @@ class MBOXImportCommand(Command):
           'help': 'skip email signature.',
           }),
         )
+    autocommit = True
+
+    def import_mbox_files(self, importer, filenames):
+        """process `filenames` with `importer` and create corresponding
+        Email / EmailThread / etc. objects in the database.
+        """
+        for fpath in filenames:
+            if fpath == '-':
+                stream = StringIO(sys.stdin.read())
+            else:
+                stream = open(fpath)
+            importer.import_mbox_stream(stream)
+        if importer.error:
+            print 'failed to import the following messages:'
+            print '\n'.join(importer.error)
+            sys.exit(1)
+
+    def connect(self, appid):
+        """create a connection to `appid`"""
+        cnx = config_connect(appid, self.config)
+        cnx.load_appobjects(cubes=None, subpath=('entities',))
+        return cnx
+
+    def create_importer(self, cnx):
+        """factory to instantiate the mbox importer"""
+        return MBOXImporter(cnx, verbose=True,
+                            interactive=self.config.interactive,
+                            skipsign=self.config.skip_sign,
+                            autocommit=self.autocommit)
+
 
     def run(self, args):
         """run the command with its specific arguments"""
-        appid = args.pop(0)
-        cnx = config_connect(appid, self.config)
-        cnx.load_appobjects(cubes=None, subpath=('entities',))
-        importer = MBOXImporter(cnx, verbose=True,
-                                interactive=self.config.interactive,
-                                skipsign=self.config.skip_sign)
-        # set autocommit, add an option to control that if needed
-        importer.autocommit_mode()
+        cnx = self.connect(args.pop(0))
+        importer = self.create_importer(cnx)
         try:
-            for fpath in args:
-                if fpath == '-':
-                    stream = StringIO(sys.stdin.read())
-                else:
-                    stream = open(fpath)
-                importer.import_mbox_stream(stream)
-            if importer.error:
-                print 'failed to import the following messages:'
-                print '\n'.join(importer.error)
-                sys.exit(1)
+            self.import_mbox_files(importer, args)
         except:
             # without a correct connection handling we exhaust repository's
             # connections pool.
             # the repository should be more resilient against bad clients !
+            cnx.rollback()
             cnx.close()
             raise
+        if not self.autocommit:
+            cnx.commit()
         cnx.close()
 
 CWCTL.register(MBOXImportCommand)
